local addonName, TS = ...; TS.UI = { }

--do not load the addon if current player is not a shaman
local a, b, classId = UnitClass("player")
if (classId ~= 7) then return end

local function CreateUiFrame(parentFrame, height)
	local frame = CreateFrame("Frame", nil, parentFrame)
	frame:SetFrameStrata("BACKGROUND")
	frame:SetWidth(175)
	frame:SetHeight(height)
	
	frame.texture = frame:CreateTexture(nil, "LOW")
	frame.texture:SetPoint("BOTTOMLEFT", frame, "BOTTOMLEFT", 3, 3);
	frame.texture:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT", 3, 3);
	frame.texture:SetPoint("TOPLEFT", frame, "TOPLEFT", 3, -3);
	frame.texture:SetPoint("TOPRIGHT", frame, "TOPRIGHT", -3, -3);
	frame.texture:SetColorTexture(0, 0, 0, 0.5)

	frame:SetBackdrop({ 
	  bgFile = nil, edgeFile = "Interface/Tooltips/UI-Tooltip-Border", 
	  tile = true, tileSize = 12, edgeSize = 12
	});
	frame:SetBackdropBorderColor(0.5, 0.5, 0.5, 1)
	
	return frame
end

local function CreateTextFrame(parentFrame, x, y, frameAnchor, textSize)
	local frame = CreateFrame("FRAME", nil, parentFrame)
	frame:SetPoint(frameAnchor, parentFrame, x, y)
	frame:SetWidth(39.375)
	frame:SetHeight(20)

	local fontString = frame:CreateFontString()
	fontString:SetAllPoints(true)
	fontString:SetJustifyH("CENTER");

	return fontString
end

local TotemFrames = { }
local function CreateSubFrame(parentFrame, x)
	local frame = CreateFrame("Button", nil, parentFrame, "ActionButtonTemplate")
	frame:SetPoint("LEFT", parentFrame, "LEFT", x, 2) 
	table.insert(TotemFrames, frame)
	frame:EnableMouse(false)
	frame:Disable()
	
	local texture = parentFrame:CreateTexture(nil, "OVERLAY")
	texture:SetAllPoints(frame)

	frame.texture = texture
	frame:SetFrameStrata("BACKGROUND")
	frame:SetWidth(36)
	frame:SetHeight(36)

	return texture
end

local function Image(frame, x)
	return CreateSubFrame(frame, x)
end

local function Timer(frame, x)
	return CreateTextFrame(frame, x, -3, "TOPLEFT", 11);
end

local function Count(frame, x)
	return CreateTextFrame(frame, x, 5, "BOTTOMLEFT", 14);
end

--create the main frame
local mainFrame = CreateUiFrame(UIParent, 85)

--create settings tooltip frame
local settingsTooltipFrame = CreateUiFrame(mainFrame, 20)
settingsTooltipFrame:SetPoint("TOP", mainFrame, 0, -83)

local fontString = settingsTooltipFrame:CreateFontString()
fontString:SetPoint("TOPLEFT", settingsTooltipFrame, 10, -5)
fontString:SetFont(STANDARD_TEXT_FONT, 10, "OUTLINE")
fontString:SetTextColor(1.0, 0.82, 0.0, 1)
fontString:SetText("Ctrl + click to open settings")

m = mainFrame
TS.UIElements = 
{ 
	[1] = { Image = Image(m, 10), Timer = Timer(m, 7), Count = Count(m, 10) }, 
	[2] = { Image = Image(m, 50), Timer = Timer(m, 47), Count = Count(m, 50) }, 
	[3] = { Image = Image(m, 90), Timer = Timer(m, 87), Count = Count(m, 90) }, 
	[4] = { Image = Image(m, 130), Timer = Timer(m, 127), Count = Count(m, 130) } 
}

local function SaveMainFrameLocation()
	local a, b, c, x, y = mainFrame:GetPoint()
	TS.Settings.SaveUILocation(x, y)
	mainFrame:StopMovingOrSizing()
end

local function MainFrameDragStart()
	mainFrame:StartMoving()
end

local function SetMovable(movable)
	mainFrame:SetMovable(movable)
	
	if (movable)
	then
		mainFrame:RegisterForDrag("LeftButton")
		mainFrame:SetScript("OnDragStart", MainFrameDragStart)
		mainFrame:SetScript("OnDragStop", SaveMainFrameLocation)
	else
		mainFrame:RegisterForDrag(nil)
		mainFrame:SetScript("OnDragStart", nil)
		mainFrame:SetScript("OnDragStop", nil)
	end
end

function TS.UI.ApplySettings(settings)
	--show/hide outer frame
	frameAlpha = 0
	if (settings.ShowOuterFrame == true) then frameAlpha = 1 end
	mainFrame:SetBackdropBorderColor(0.5, 0.5, 0.5, frameAlpha)
	
	--show/hide totem frames
	for key, frame in pairs(TotemFrames)
	do
		if (settings.ShowTotemFrames) then frame:Show() else frame:Hide() end
	end

	--lock location
	SetMovable(not settings.LockLocation)	

	--font & font size
	for key, value in pairs(TS.UIElements)
	do
		value.Timer:SetFont(settings.Font, settings.FontSize - 1.5, "OUTLINE")
		value.Count:SetFont(settings.Font, settings.FontSize + 1.5, "OUTLINE")
	end

	--UI Scale
	mainFrame:SetScale(settings.UIScale)
	
	--color & transparency
	local color = settings.Color
	mainFrame.texture:SetColorTexture(color.R, color.G, color.B, settings.Transparency)
end

function TS.UI.SetLocation(x, y)
	mainFrame:ClearAllPoints();
	
	if (x == nil or y == nil) 
	then
		--if location is not given in settings we want to show the UI in the center of the screen
		mainFrame:SetPoint("CENTER", 0, 0)
	else
		mainFrame:SetPoint("TOPLEFT", x, y)
	end
end

local function AddonLoaded(addonName)
	if (addonName ~= "TotemStatus") then return end
	
	TS.Settings.LoadSettings()
	TS.Logic.UpdateTotemData()
	TS.UI.SetLocation(TS_Settings.MainFrameX, TS_Settings.MainFrameY)
end

function mainFrame:OnEvent(eventName, arg1, arg2, arg3)
	if eventName == "ADDON_LOADED" then AddonLoaded(arg1) end
	if eventName == "ZONE_CHANGED_NEW_AREA" then TS.Logic.ZoneChanged() end
	if eventName == "PLAYER_TOTEM_UPDATE" then TS.Logic.PlayerTotemUpdate(arg1) end
	if eventName == "UNIT_SPELLCAST_SUCCEEDED" then TS.Logic.SpellCastSucceeded(arg3) end
end

local function OnMouseDown()
	if (IsControlKeyDown()) then TS.Settings.ShowSettingsDialog() end
end

--[[
	Show/hide settings tooltip frame and enable/disable mouse.
	We shall enable the mouse only when mouse is over mainFrame AND the Ctrl key is down.
	Like this settings will open with Ctrl + Click while keeping the frame click-through at all other times.
]]
local function OnUpdate(self, elapsed)
	if (mainFrame:IsMouseOver())
	then
		settingsTooltipFrame:Show()
		mainFrame:EnableMouse(IsControlKeyDown())
	else
		settingsTooltipFrame:Hide()
		mainFrame:EnableMouse(false)
	end
	
	TS.Logic.OnUpdate(elapsed)
end

--register events and handlers
mainFrame:RegisterEvent("ADDON_LOADED")
mainFrame:RegisterEvent("PLAYER_TOTEM_UPDATE")
mainFrame:RegisterEvent("ZONE_CHANGED_NEW_AREA")
mainFrame:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED")
mainFrame:SetScript("OnUpdate", OnUpdate)
mainFrame:SetScript("OnEvent", mainFrame.OnEvent)
mainFrame:SetScript("OnMouseDown", OnMouseDown)

--disable mouse (clickthrough) and enable keyboard
mainFrame:EnableMouse(false)
mainFrame:EnableKeyboard(true)
mainFrame:SetPropagateKeyboardInput(true)

mainFrame:Show()