local addonName, TS = ...; TS.Utils = { }

--[[
	Join 2 arrays into a new single array.
]]
local function Join(arr1, arr2)
	local newArray = { }

	for _, value in pairs(arr1) do
		table.insert(newArray, value)
	end
	
	if (arr2 == nil) then return newArray end
	
	for _, value in pairs(arr2) do
		table.insert(newArray, value)
	end
	
	return newArray
end

--[[
	Join alll the provided arrays into 1. Any parameter past the first can be nil.
]]
local function JoinMany(arr1, arr2, arr3, arr4, arr5, arr6, arr7)
	return Join(Join(Join(Join(Join(Join(arr1, arr2), arr3), arr4), arr5), arr6), arr7)
end

--Each array contains all the aura spell IDs of all ranks for that totem
local FIRE_NOVA = { 1535, 8498, 8499, 11314, 11315 }
local FIRE_RES = { 8185, 10534, 10535 }
local FROST_RES = { 8182, 10476, 10477 }
local GRACE_OF_AIR = { 8836, 10626, 25360 }
local GROUNDING = { 8178 }
local NATURE_RES = { 10596, 10598, 10599 }
local STONESKIN = { 8072, 8156, 8157, 10403, 10404, 10405 }
local STRENGTH_OF_EARTH = { 8076, 8162, 8163, 10441, 25362 }
local WINDWALL = { 15108, 15109, 15110 }
local HEALING_STREAM = { 5672, 6371, 6372, 10460, 10461 }
local MANA_SPRING = { 5677, 10491, 10493, 10494 }
local MANA_TIDE = { 16191, 17355, 17360 }
local TRANQUIL_AIR = { 25909 }
local ALL_BENEFIT = JoinMany(FIRE_RES, FROST_RES, GROUNDING, NATURE_RES, WINDWALL, HEALING_STREAM, TRANQUIL_AIR)

--Defines which totems are considered useful to which classes
local ClassUsefulTotemAuras = 
{ 
	[1] = JoinMany(ALL_BENEFIT, GRACE_OF_AIR, STONESKIN, STRENGTH_OF_EARTH), --Warrior
	[3] = JoinMany(ALL_BENEFIT, GRACE_OF_AIR, MANA_SPRING, MANA_TIDE), --Hunter
	[4] = JoinMany(ALL_BENEFIT, GRACE_OF_AIR, STONESKIN, STRENGTH_OF_EARTH), --Rogue
	[5] = JoinMany(ALL_BENEFIT, MANA_SPRING, MANA_TIDE), --Priest
	[7] = JoinMany(ALL_BENEFIT, GRACE_OF_AIR, STONESKIN, STRENGTH_OF_EARTH, MANA_SPRING, MANA_TIDE), --Shaman
	[8] = JoinMany(ALL_BENEFIT, MANA_SPRING, MANA_TIDE), --Mage
	[9] = JoinMany(ALL_BENEFIT, MANA_SPRING, MANA_TIDE), --Warlock 
	[11] = JoinMany(ALL_BENEFIT, GRACE_OF_AIR, STONESKIN, STRENGTH_OF_EARTH, MANA_SPRING, MANA_TIDE) --Druid 
}

--calculate Fire Nova totem duration based on if the "Improved Fire Totems" talent is selected
local a, b, c, d, improvedFireTotemsTalentRank = GetTalentInfo(1, 9);
local FireNovaDuration = (5 - improvedFireTotemsTalentRank)

local function Totem(totemIndex, auraId, duration)
	return { TotemIndex = totemIndex, AuraId = auraId, Duration = duration }
end

TS.Totems = 
{
	[2484] = Totem(1, 0, 45),			--Earthbind
	
	[1535] = Totem(2, 0, FireNovaDuration),		--Fire Nova Rank 1
	[8498] = Totem(2, 0, FireNovaDuration),		--Fire Nova Rank 2
	[8499] = Totem(2, 0, FireNovaDuration),		--Fire Nova Rank 3
	[11314] = Totem(2, 0, FireNovaDuration),	--Fire Nova Rank 4
	[11315] = Totem(2, 0, FireNovaDuration),	--Fire Nova Rank 5
	
	[8190] = Totem(2, 0, 20),			--Magma Rank 1
	[10585] = Totem(2, 0, 20),			--Magma Rank 2
	[10586] = Totem(2, 0, 20),			--Magma Rank 3
	[10587] = Totem(2, 0, 20),			--Magma Rank 4

	[3599] = Totem(2, 0, 30),			--Searing Rank 1
	[6363] = Totem(2, 0, 35),			--Searing Rank 2
	[6364] = Totem(2, 0, 40),			--Searing Rank 3
	[6365] = Totem(2, 0, 45),			--Searing Rank 4
	[10437] = Totem(2, 0, 50),			--Searing Rank 5
	[10438] = Totem(2, 0, 55),			--Searing Rank 6
	
	[5730] = Totem(1, 0, 15),			--Stoneclaw Rank 1
	[6390] = Totem(1, 0, 15),			--Stoneclaw Rank 2
	[6391] = Totem(1, 0, 15),			--Stoneclaw Rank 3
	[6392] = Totem(1, 0, 15),			--Stoneclaw Rank 4
	[10427] = Totem(1, 0, 15),			--Stoneclaw Rank 5
	[10428] = Totem(1, 0, 15),			--Stoneclaw Rank 6
	
	[8184] = Totem(3, 8185, 120),		--Fire Resistance Rank 1
	[10537] = Totem(3, 10534, 120), 	--Fire Resistance Rank 2
	[10538] = Totem(3, 10535, 120), 	--Fire Resistance Rank 3
	
	[8227] = Totem(2, 0, 120),			--Flametongue Rank 1
	[8249] = Totem(2, 0, 120),			--Flametongue Rank 2
	[10526] = Totem(2, 0, 120),  		--Flametongue Rank 3
	[16387] = Totem(2, 0, 120),  		--Flametongue Rank 4
	
	[8181] = Totem(2, 8182, 120),		--Frost Resistance Rank 1
	[10478] = Totem(2, 10476, 120),  	--Frost Resistance Rank 2
	[10479] = Totem(2, 10477, 120),  	--Frost Resistance Rank 3
	
	[8835] = Totem(4, 8836, 120),  		--Grace of Air Rank 1
	[10627] = Totem(4, 10626, 120),  	--Grace of Air Rank 2
	[25359] = Totem(4, 25360, 120),  	--Grace of Air Rank 3
	
	[8177] = Totem(4, 8178, 45), 		--Grounding
	
	[10595] = Totem(4, 10596, 120),  	--Nature Resistance Rank 1
	[10600] = Totem(4, 10598, 120),  	--Nature Resistance Rank 2
	[10601] = Totem(4, 10599, 120),  	--Nature Resistance Rank 3
	
	[6495] = Totem(4, 0, 300),  		--Sentry
	
	[8071] = Totem(1, 8072, 120),  		--Stoneskin Rank 1
	[8154] = Totem(1, 8156, 120),  		--Stoneskin Rank 2
	[8155] = Totem(1, 8157, 120),  		--Stoneskin Rank 3
	[10406] = Totem(1, 10403, 120),  	--Stoneskin Rank 4
	[10407] = Totem(1, 10404, 120),  	--Stoneskin Rank 5
	[10408] = Totem(1, 10405, 120),  	--Stoneskin Rank 6
	
	[8075] = Totem(1, 8076, 120),  		--Strenth of Earth Rank 1
	[8160] = Totem(1, 8162, 120),  		--Strenth of Earth Rank 2
	[8161] = Totem(1, 8163, 120),  		--Strenth of Earth Rank 3
	[10442] = Totem(1, 10441, 120),  	--Strenth of Earth Rank 4
	[25361] = Totem(1, 25362, 120),  	--Strenth of Earth Rank 5

	[8512] = Totem(4, 0, 120) ,  		--Windfury Rank 1
	[10613] = Totem(4, 0, 120) ,  		--Windfury Rank 2
	[10614] = Totem(4, 0, 120) ,  		--Windfury Rank 3
	
	[15107] = Totem(4, 15108, 120),  	--Windwall Rank 1
	[15111] = Totem(4, 15109, 120),  	--Windwall Rank 2
	[15112] = Totem(4, 15110, 120),  	--Windwall Rank 3
	
	[8170] = Totem(3, 0, 120),  		--Disease Cleansing
	
	[5394] = Totem(3, 5672, 60),  		--Healing Stream Rank 1
	[6375] = Totem(3, 6371, 60),  		--Healing Stream Rank 2
	[6377] = Totem(3, 6372, 60),  		--Healing Stream Rank 3
	[10462] = Totem(3, 10460, 60),  	--Healing Stream Rank 4
	[10463] = Totem(3, 10461, 60),  	--Healing Stream Rank 5
	
	[5675] = Totem(3, 5677, 60),  		--Mana Spring Rank 1
	[10495] = Totem(3, 10491, 60),  	--Mana Spring Rank 2
	[10496] = Totem(3, 10493, 60),  	--Mana Spring Rank 3
	[10497] = Totem(3, 10494, 60),  	--Mana Spring Rank 4

	[16190] = Totem(3, 16191, 12),  	--Mana Tide Rank 1
	[17354] = Totem(3, 17355, 12),  	--Mana Tide Rank 2
	[17359] = Totem(3, 17360, 12),  	--Mana Tide Rank 3
	
	[8166] = Totem(3, 0, 120),  		--Poison Cleansing

	[25908] = Totem(4, 25909, 120),  	--Tranquil Air
	
	[8143] = Totem(1, 0, 120)  			--Tremor
}

--[[
	Determines if the totem aura with the given aura spell ID 
	is useful to the unit with the given unit ID.
]]
function TS.Utils.IsAuraUsefulToUnit(unitId, spellId)
	local a, b, classId = UnitClass(unitId)
	
	--find the useful auras for this class
	local usefulAuras = ClassUsefulTotemAuras[classId]
	if (usefulAuras == nil) then return end
	
	--loop over all the values to see if it contains our spell ID
	for _, value in pairs(usefulAuras) do
		if value == spellId then return true end
	end
	
	return false
end

function TS.Utils.GetSpellTotemIndex(spellId)
	if (TS.Totems[spellId] == nil) then return nil end
	
	return TS.Totems[spellId].TotemIndex
end

--[[
	Get the unit ID string used by the Blizzard API for the given unit index (1-5).
	1 = player, 2, 3, 4, 5 = party members
]]
function TS.Utils.GetUnitId(unitIndex)
	if (unitIndex == 1) then return "player" end
	
	return "party" .. (unitIndex - 1)
end

--[[
	Determine if the unit with the given ID is affected with the given spell ID.
]]
function TS.Utils.IsUnitAffected(unitId, spellId)
	for buffIndex = 1, 40 
	do
		--get the spell ID for this buff index
		local a, b, c, d, e, f, g, h, i, unitSpellId = UnitAura(unitId, buffIndex, "HELPFUL")
		if (unitSpellId == spellId) then return true end
	end
	
	return false
end

--[[
	Determine if the unit is able to receive totem buffs.
]]
function TS.Utils.CanUnitReceiveTotemBuffs(unitId)
    return (UnitIsConnected(unitId) and UnitExists(unitId) and not UnitIsDeadOrGhost(unitId))
end

--[[
	Determines if the totem with the provided spell ID should be ignored in the expiry alarm.
]]
function TS.Utils.IsTotemIgnoredForAlarm(spellId)
	for index, value in ipairs(FIRE_NOVA) do
        if spellId == value then return true end
    end

    return false
end